@@ -56,7 +56,7 @@ struct _GMimeStreamBuffer {
 	unsigned char *buffer;
 	unsigned char *bufptr;
 	unsigned char *bufend;
-	size_t buflen;
+	ssize_t buflen;
 	
 	GMimeStreamBufferMode mode;
 };
