@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,15 +50,16 @@ struct _GMimeStreamFilterPrivate {
 	char *filtered;		/* the filtered data */
 	size_t filteredlen;
 	
-	int last_was_read;	/* was the last op read or write? */
+	int last_was_read:1;	/* was the last op read or write? */
+	int flushed:1;          /* have the filters been flushed? */
 };
 
 static void g_mime_stream_filter_class_init (GMimeStreamFilterClass *klass);
 static void g_mime_stream_filter_init (GMimeStreamFilter *stream, GMimeStreamFilterClass *klass);
 static void g_mime_stream_filter_finalize (GObject *object);
 
-static ssize_t stream_read (GMimeStream *stream, char *buf, size_t len);
-static ssize_t stream_write (GMimeStream *stream, char *buf, size_t len);
+static ssize_t stream_read (GMimeStream *stream, char *buf, size_t n);
+static ssize_t stream_write (GMimeStream *stream, const char *buf, size_t n);
 static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
@@ -130,6 +131,7 @@ g_mime_stream_filter_init (GMimeStreamFilter *stream, GMimeStreamFilterClass *kl
 	stream->priv->buffer = stream->priv->realbuffer + READ_PAD;
 	stream->priv->last_was_read = TRUE;
 	stream->priv->filteredlen = 0;
+	stream->priv->flushed = FALSE;
 }
 
 static void
@@ -151,14 +153,14 @@ g_mime_stream_filter_finalize (GObject *object)
 	g_free (p);
 	
 	if (filter->source)
-		g_mime_stream_unref (filter->source);
+		g_object_unref (filter->source);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
 static ssize_t
-stream_read (GMimeStream *stream, char *buf, size_t len)
+stream_read (GMimeStream *stream, char *buf, size_t n)
 {
 	GMimeStreamFilter *filter = (GMimeStreamFilter *) stream;
 	struct _GMimeStreamFilterPrivate *p = filter->priv;
@@ -184,6 +186,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 					f = f->next;
 				}
 				size = p->filteredlen;
+				p->flushed = TRUE;
 			}
 			if (size <= 0)
 				return size;
@@ -201,7 +204,7 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		}
 	}
 	
-	size = MIN (len, p->filteredlen);
+	size = MIN (n, p->filteredlen);
 	memcpy (buf, p->filtered, size);
 	p->filteredlen -= size;
 	p->filtered += size;
@@ -210,20 +213,17 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 }
 
 static ssize_t
-stream_write (GMimeStream *stream, char *buf, size_t len)
+stream_write (GMimeStream *stream, const char *buf, size_t n)
 {
 	GMimeStreamFilter *filter = (GMimeStreamFilter *) stream;
 	struct _GMimeStreamFilterPrivate *p = filter->priv;
+	char *buffer = (char *) buf;
+	ssize_t nwritten = n;
 	struct _filter *f;
 	size_t presize;
-	char *buffer;
-	size_t n;
 	
 	p->last_was_read = FALSE;
 	
-	buffer = buf;
-	n = len;
-	
 	f = p->filters;
 	presize = 0;
 	while (f) {
@@ -235,8 +235,8 @@ stream_write (GMimeStream *stream, char *buf, size_t len)
 	if (g_mime_stream_write (filter->source, buffer, n) != n)
 		return -1;
 	
-	/* return 'len' because that's what our caller expects */
-	return len;
+	/* return original input len because that's what our caller expects */
+	return nwritten;
 }
 
 static int
@@ -292,6 +292,9 @@ stream_eos (GMimeStream *stream)
 	if (p->filteredlen > 0)
 		return FALSE;
 	
+	if (!p->flushed)
+		return FALSE;
+	
 	return g_mime_stream_eos (filter->source);
 }
 
@@ -303,6 +306,7 @@ stream_reset (GMimeStream *stream)
 	struct _filter *f;
 	
 	p->filteredlen = 0;
+	p->flushed = FALSE;
 	
 	/* and reset filters */
 	f = p->filters;
@@ -343,7 +347,7 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
 	
 	sub = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL, NULL);
 	sub->source = filter->source;
-	g_mime_stream_ref (sub->source);
+	g_object_ref (sub->source);
 	
 	if (filter->priv->filters) {
 		struct _filter *f, *sn, *s = NULL;
@@ -395,7 +399,7 @@ g_mime_stream_filter_new_with_stream (GMimeStream *stream)
 	
 	filter = g_object_new (GMIME_TYPE_STREAM_FILTER, NULL, NULL);
 	filter->source = stream;
-	g_mime_stream_ref (stream);
+	g_object_ref (stream);
 	
 	g_mime_stream_construct (GMIME_STREAM (filter),
 				 stream->bound_start,
