@@ -30,17 +30,19 @@
 #include <unistd.h>
 #include <stdarg.h>
 
+#include <gmime/gmime-type-utils.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif /* __cplusplus */
 
 #define GMIME_TYPE_STREAM            (g_mime_stream_get_type ())
-#define GMIME_STREAM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_STREAM, GMimeStream))
-#define GMIME_STREAM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_STREAM, GMimeStreamClass))
-#define GMIME_IS_STREAM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_STREAM))
-#define GMIME_IS_STREAM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_STREAM))
-#define GMIME_STREAM_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_STREAM, GMimeStreamClass))
+#define GMIME_STREAM(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_STREAM, GMimeStream))
+#define GMIME_STREAM_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_STREAM, GMimeStreamClass))
+#define GMIME_IS_STREAM(obj)         (GMIME_CHECK_TYPE ((obj), GMIME_TYPE_STREAM))
+#define GMIME_IS_STREAM_CLASS(klass) (GMIME_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_STREAM))
+#define GMIME_STREAM_GET_CLASS(obj)  (GMIME_CHECK_GET_CLASS ((obj), GMIME_TYPE_STREAM, GMimeStreamClass))
 
 typedef enum {
 	GMIME_STREAM_SEEK_SET = SEEK_SET,
@@ -70,16 +72,16 @@ struct _GMimeStream {
 struct _GMimeStreamClass {
 	GObjectClass parent_class;
 	
-	ssize_t  (* read)   (GMimeStream *stream, char *buf, size_t len);
-	ssize_t  (* write)  (GMimeStream *stream, const char *buf, size_t len);
-	int      (* flush)  (GMimeStream *stream);
-	int      (* close)  (GMimeStream *stream);
-	gboolean (* eos)    (GMimeStream *stream);
-	int      (* reset)  (GMimeStream *stream);
-	off_t    (* seek)   (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
-	off_t    (* tell)   (GMimeStream *stream);
-	ssize_t  (* length) (GMimeStream *stream);
-	GMimeStream * (* substream) (GMimeStream *stream, off_t start, off_t end);
+	ssize_t  (*read)    (GMimeStream *stream, char *buf, size_t len);
+	ssize_t  (*write)   (GMimeStream *stream, const char *buf, size_t len);
+	int      (*flush)   (GMimeStream *stream);
+	int      (*close)   (GMimeStream *stream);
+	gboolean (*eos)     (GMimeStream *stream);
+	int      (*reset)   (GMimeStream *stream);
+	off_t    (*seek)    (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
+	off_t    (*tell)    (GMimeStream *stream);
+	ssize_t  (*length)  (GMimeStream *stream);
+	GMimeStream *(*substream) (GMimeStream *stream, off_t start, off_t end);
 };
 
 
