@@ -50,6 +50,7 @@
 #include "gmime-charset.h"
 #include "gmime-iconv.h"
 #include "gmime-iconv-utils.h"
+#include "gmime-host-utils.h"
 
 #ifndef HAVE_ISBLANK
 #define isblank(c) (c == ' ' || c == '\t')
@@ -463,32 +464,6 @@ get_tzone (struct _date_token **token)
 	return -1;
 }
 
-static time_t
-mktime_utc (struct tm *tm)
-{
-	time_t tt;
-	
-	tm->tm_isdst = -1;
-	tt = mktime (tm);
-	
-#if defined (HAVE_TM_GMTOFF)
-	tt += tm->tm_gmtoff;
-#elif defined (HAVE_TIMEZONE)
-	if (tm->tm_isdst > 0) {
-#if defined (HAVE_ALTZONE)
-		tt -= altzone;
-#else /* !defined (HAVE_ALTZONE) */
-		tt -= (timezone - 3600);
-#endif
-	} else
-		tt -= timezone;
-#else
-#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
-#endif
-	
-	return tt;
-}
-
 static time_t
 parse_rfc822_date (struct _date_token *tokens, int *tzone)
 {
@@ -547,7 +522,22 @@ parse_rfc822_date (struct _date_token *tokens, int *tzone)
 		offset = n;
 	}
 	
-	t = mktime_utc (&tm);
+	t = mktime (&tm);
+#if defined (HAVE_TM_GMTOFF)
+	t += tm.tm_gmtoff;
+#elif defined (HAVE_TIMEZONE)
+	if (tm.tm_isdst > 0) {
+#if defined (HAVE_ALTZONE)
+		t -= altzone;
+#else /* !defined (HAVE_ALTZONE) */
+		t -= (timezone - 3600);
+#endif
+	} else {
+		t -= timezone;
+	}
+#else
+#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
+#endif
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
@@ -659,8 +649,23 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	}
 	
 	d(printf ("\n"));
-	
-	t = mktime_utc (&tm);
+		
+	t = mktime (&tm);
+#if defined (HAVE_TM_GMTOFF)
+	t += tm.tm_gmtoff;
+#elif defined (HAVE_TIMEZONE)
+	if (tm.tm_isdst > 0) {
+#if defined (HAVE_ALTZONE)
+		t -= altzone;
+#else /* !defined (HAVE_ALTZONE) */
+		t -= (timezone - 3600);
+#endif
+	} else {
+		t -= timezone;
+	}
+#else
+#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
+#endif
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
@@ -736,12 +741,7 @@ g_mime_utils_header_decode_date (const char *in, int *saveoffset)
 	struct _date_token *token, *tokens;
 	time_t date;
 	
-	if (!(tokens = datetok (in))) {
-		if (saveoffset)
-			*saveoffset = 0;
-		
-		return (time_t) 0;
-	}
+	tokens = datetok (in);
 	
 	date = parse_rfc822_date (tokens, saveoffset);
 	if (!date)
@@ -780,25 +780,29 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #endif
 	static unsigned int count = 0;
 	char host[MAXHOSTNAMELEN + 1];
-#ifdef HAVE_GETADDRINFO
-	struct addrinfo hints, *res;
-	char *name = NULL;
-	int ret;
+#ifdef HAVE_GETHOSTBYNAME
+	struct hostent hostbuf;
+	char *buf = NULL;
 #endif
 	char *msgid;
 	
 	if (!fqdn) {
 #ifdef HAVE_GETHOSTNAME
 		if (gethostname (host, sizeof (host)) == 0) {
-#ifdef HAVE_GETADDRINFO
-			memset (&hints, 0, sizeof (hints));
-			hints.ai_flags = AI_CANONNAME;
+#ifdef HAVE_GETHOSTBYNAME
+			size_t buflen = 1024;
+			int retval;
 			
-			if (getaddrinfo (host, NULL, &hints, &res) == 0) {
-				name = g_strdup (res->ai_canonname);
-				freeaddrinfo (res);
+			buf = g_malloc (buflen);
+			
+			while ((retval = g_gethostbyname_r (host, &hostbuf, buf, buflen, NULL)) == ERANGE && buflen < 8192)
+				buf = g_realloc (buf, (buflen += 1024));
+			
+			if (retval == -1) {
+				g_free (buf);
+				buf = NULL;
 			}
-#endif /* HAVE_GETADDRINFO */
+#endif /* HAVE_GETHOSTBYNAME */
 		} else {
 			host[0] = '\0';
 		}
@@ -806,9 +810,9 @@ g_mime_utils_generate_message_id (const char *fqdn)
 		host[0] = '\0';
 #endif /* HAVE_GETHOSTNAME */
 		
-#ifdef HAVE_GETADDRINFO
-		fqdn = name != NULL ? name : (*host ? host : "localhost.localdomain");
-		g_free (name);
+#ifdef HAVE_GETHOSTBYNAME
+		fqdn = buf != NULL ? hostbuf.h_name : (*host ? host : "localhost.localdomain");
+		g_free (buf);
 #else
 		fqdn = *host ? host : "localhost.localdomain";
 #endif
@@ -834,30 +838,23 @@ decode_msgid (const char **in)
 	char *msgid = NULL;
 	
 	decode_lwsp (&inptr);
-	if (*inptr != '<') {
-		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
-	} else {
+	if (*inptr == '<') {
 		inptr++;
-	}
-	
-	decode_lwsp (&inptr);
-	if ((msgid = decode_addrspec (&inptr))) {
 		decode_lwsp (&inptr);
-		if (*inptr != '>') {
-			w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+		if ((msgid = decode_addrspec (&inptr))) {
+			decode_lwsp (&inptr);
+			if (*inptr != '>') {
+				w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+			} else {
+				inptr++;
+			}
+			
+			*in = inptr;
 		} else {
-			inptr++;
+			w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
 		}
-		
-		*in = inptr;
 	} else {
-		w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
-		*in = inptr;
-		while (*inptr && *inptr != '>')
-			inptr++;
-		
-		msgid = g_strndup (*in, inptr - *in);
-		*in = inptr;
+		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
 	}
 	
 	return msgid;
@@ -981,161 +978,76 @@ g_mime_references_clear (GMimeReferences **refs)
 }
 
 
-static gboolean
-is_rfc2047_token (const char *inptr, size_t len)
-{
-	if (len < 8 || strncmp (inptr, "=?", 2) != 0 || strncmp (inptr + len - 2, "?=", 2) != 0)
-		return FALSE;
-	
-	inptr += 3;
-	len -= 3;
-	
-	while (*inptr != '?' && len > 0) {
-		inptr++;
-		len--;
-	}
-	
-	if (*inptr != '?' || len < 4)
-		return FALSE;
-	
-	if (inptr[1] != 'q' && inptr[1] != 'Q' && inptr[1] != 'b' && inptr[1] != 'B')
-		return FALSE;
-	
-	inptr += 2;
-	len -= 2;
-	
-	if (*inptr != '?')
-		return FALSE;
-	
-	return TRUE;
-}
-
-static char *
-header_fold (const char *in, gboolean structured)
-{
-	gboolean last_was_lwsp = FALSE;
-	register const char *inptr;
-	size_t len, outlen, i;
-	size_t fieldlen;
-	GString *out;
-	char *ret;
-	
-	inptr = in;
-	len = strlen (in);
-	if (len <= GMIME_FOLD_LEN + 1)
-		return g_strdup (in);
-	
-	out = g_string_new ("");
-	fieldlen = strcspn (inptr, ": \t\n");
-	g_string_append_len (out, inptr, fieldlen);
-	outlen = fieldlen;
-	inptr += fieldlen;
-	
-	while (*inptr && *inptr != '\n') {
-		len = strcspn (inptr, " \t\n");
-		
-		if (len > 1 && outlen + len > GMIME_FOLD_LEN) {
-			if (outlen > 1 && out->len > fieldlen + 2) {
-				if (last_was_lwsp) {
-					if (structured)
-						out->str[out->len - 1] = '\t';
-					
-					g_string_insert_c (out, out->len - 1, '\n');
-				} else
-					g_string_append (out, "\n\t");
-				outlen = 1;
-			}
-			
-			if (!structured && !is_rfc2047_token (inptr, len)) {
-				/* check for very long words, just cut them up */
-				while (outlen + len > GMIME_FOLD_LEN) {
-					for (i = 0; i < GMIME_FOLD_LEN - outlen; i++)
-						g_string_append_c (out, inptr[i]);
-					inptr += GMIME_FOLD_LEN - outlen;
-					len -= GMIME_FOLD_LEN - outlen;
-					g_string_append (out, "\n\t");
-					outlen = 1;
-				}
-			} else {
-				g_string_append_len (out, inptr, len);
-				outlen += len;
-				inptr += len;
-			}
-			last_was_lwsp = FALSE;
-		} else if (len > 0) {
-			g_string_append_len (out, inptr, len);
-			outlen += len;
-			inptr += len;
-			last_was_lwsp = FALSE;
-		} else {
-			last_was_lwsp = TRUE;
-			if (*inptr == '\t') {
-				/* tabs are a good place to fold, odds
-				   are that this is where the previous
-				   mailer folded it */
-				g_string_append (out, "\n\t");
-				outlen = 1;
-				while (isblank (*inptr))
-					inptr++;
-			} else {
-				g_string_append_c (out, *inptr++);
-				outlen++;
-			}
-		}
-	}
-	
-	if (*inptr == '\n' && out->str[out->len - 1] != '\n')
-		g_string_append_c (out, '\n');
-	
-	ret = out->str;
-	g_string_free (out, FALSE);
-	
-	return ret;
-}
-
-
-/**
- * g_mime_utils_structured_header_fold:
- * @in: input header string
- *
- * Folds a structured header according to the rules in rfc822.
- *
- * Returns an allocated string containing the folded header.
- **/
-char *
-g_mime_utils_structured_header_fold (const char *in)
-{
-	return header_fold (in, TRUE);
-}
-
-
-/**
- * g_mime_utils_unstructured_header_fold:
- * @in: input header string
- *
- * Folds an unstructured header according to the rules in rfc822.
- *
- * Returns an allocated string containing the folded header.
- **/
-char *
-g_mime_utils_unstructured_header_fold (const char *in)
-{
-	return header_fold (in, FALSE);
-}
-
-
 /**
  * g_mime_utils_header_fold:
  * @in: input header string
  *
- * Folds a structured header according to the rules in rfc822.
+ * Folds a header according to the rules in rfc822.
  *
  * Returns an allocated string containing the folded header.
  **/
 char *
 g_mime_utils_header_fold (const char *in)
 {
-	return header_fold (in, TRUE);
+	gboolean last_was_lwsp = FALSE;
+	register const char *inptr;
+	size_t len, outlen, i;
+	GString *out;
+	char *ret;
+	
+	inptr = in;
+	len = strlen (in);
+	if (len <= GMIME_FOLD_LEN)
+		return g_strdup (in);
+	
+	out = g_string_new ("");
+	outlen = 0;
+	while (*inptr) {
+		len = strcspn (inptr, " \t");
+		
+		if (outlen + len > GMIME_FOLD_LEN) {			
+			if (last_was_lwsp)
+				g_string_truncate (out, out->len - 1);
+			
+			g_string_append (out, "\n\t");
+			outlen = 1;
+			
+			/* check for very long words, just cut them up */
+			while (outlen + len > GMIME_FOLD_LEN) {
+				for (i = 0; i < GMIME_FOLD_LEN - outlen; i++)
+					g_string_append_c (out, inptr[i]);
+				inptr += GMIME_FOLD_LEN - outlen;
+				len -= GMIME_FOLD_LEN - outlen;
+				g_string_append (out, "\n\t");
+				outlen = 1;
+			}
+			last_was_lwsp = FALSE;
+		} else if (len > 0) {
+			outlen += len;
+			g_string_append_len (out, inptr, len);
+			inptr += len;
+			last_was_lwsp = FALSE;
+		} else {
+			if (*inptr == '\t') {
+				/* tabs are a good place to fold, odds
+                                   are that this is where the previous
+                                   mailer folded it */
+				g_string_append (out, "\n\t");
+				outlen = 1;
+				inptr++;
+				last_was_lwsp = FALSE;
+			} else {
+				g_string_append_c (out, *inptr++);
+				outlen++;
+				last_was_lwsp = TRUE;
+			}
+		}
+	}
+	
+	ret = out->str;
+	g_string_free (out, FALSE);
+	
+	return ret;
 }
 
 
@@ -1160,7 +1072,7 @@ g_mime_utils_header_printf (const char *format, ...)
 	buf = g_strdup_vprintf (format, ap);
 	va_end (ap);
 	
-	ret = header_fold (buf, TRUE);
+	ret = g_mime_utils_header_fold (buf);
 	g_free (buf);
 	
 	return ret;
@@ -1841,7 +1753,7 @@ rfc2047_encode_get_rfc822_words (const unsigned char *in, gboolean phrase)
 		
 		inptr = newinptr;
 		
-		if (c < 256 && is_lwsp (c)) {
+		if (g_unichar_isspace (c)) {
 			if (count > 0) {
 				word = g_new (struct _rfc822_word, 1);
 				word->next = NULL;
@@ -1871,23 +1783,6 @@ rfc2047_encode_get_rfc822_words (const unsigned char *in, gboolean phrase)
 				type = WORD_2047;
 				encoding = 2;
 			}
-			
-			if (count >= GMIME_FOLD_PREENCODED) {
-				word = g_new (struct _rfc822_word, 1);
-				word->next = NULL;
-				word->start = start;
-				word->end = inptr;
-				word->type = type;
-				word->encoding = encoding;
-				
-				tail->next = word;
-				tail = word;
-				count = 0;
-				
-				start = inptr;
-				type = WORD_ATOM;
-				encoding = 0;
-			}
 		}
 		
 		last = inptr;
@@ -2093,7 +1988,7 @@ g_mime_utils_header_encode_text (const unsigned char *in)
  *
  * Decodes an rfc2047 encoded header.
  *
- * WARNING: This function is deprecated. Use
+ * Note: This function is deprecated. Use
  * #g_mime_utils_header_decode_text instead.
  *
  * Returns the decoded header (which will be in UTF-8 if at all
@@ -2112,7 +2007,7 @@ g_mime_utils_8bit_header_decode (const unsigned char *in)
  *
  * Encodes a 'text' header according to the rules in rfc2047.
  *
- * WARNING: This function is deprecated. Use
+ * Note: This function is deprecated. Use
  * #g_mime_utils_header_encode_text instead.
  *
  * Returns the encoded header. Useful for encoding
@@ -2131,7 +2026,7 @@ g_mime_utils_8bit_header_encode (const unsigned char *in)
  *
  * Encodes a 'phrase' header according to the rules in rfc2047.
  *
- * WARNING: This function is deprecated. Use
+ * Note: This function is deprecated. Use
  * #g_mime_utils_header_encode_phrase instead.
  *
  * Returns the encoded 'phrase'. Useful for encoding internet
@@ -2512,7 +2407,7 @@ g_mime_utils_uuencode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been decoded
  *
  * Uudecodes a chunk of data. Performs a 'decode step' on a chunk of
- * uuencoded data. Assumes the "begin mode filename" line has
+ * uuencoded data. Assumes the "begin <mode> <file name>" line has
  * been stripped off.
  *
  * Returns the number of bytes decoded.
@@ -2747,7 +2642,7 @@ g_mime_utils_quoted_encode_step (const unsigned char *in, size_t inlen, unsigned
 
 
 /**
- * g_mime_utils_quoted_decode_step:
+ * g_mime_utils_quoted_decode_step: decode a chunk of QP encoded data
  * @in: input stream
  * @inlen: max length of data to decode
  * @out: output stream
