@@ -49,6 +49,7 @@ extern "C" {
 #include <gmime/internet-address.h>
 #include <gmime/gmime-parser.h>
 #include <gmime/gmime-utils.h>
+#include <gmime/gmime-host-utils.h>
 #include <gmime/gmime-stream.h>
 #include <gmime/gmime-stream-buffer.h>
 #include <gmime/gmime-stream-cat.h>
@@ -65,21 +66,19 @@ extern "C" {
 #include <gmime/gmime-filter-crlf.h>
 #include <gmime/gmime-filter-enriched.h>
 #include <gmime/gmime-filter-from.h>
-#include <gmime/gmime-filter-gzip.h>
 #include <gmime/gmime-filter-html.h>
 #include <gmime/gmime-filter-md5.h>
 #include <gmime/gmime-filter-strip.h>
 #include <gmime/gmime-filter-windows.h>
 #include <gmime/gmime-filter-yenc.h>
 #include <gmime/gmime-session.h>
-#include <gmime/gmime-session-simple.h>
 #include <gmime/gmime-cipher-context.h>
 #include <gmime/gmime-gpg-context.h>
 
 /* GMIME version */
 static const guint gmime_major_version = 2;
 static const guint gmime_minor_version = 1;
-static const guint gmime_micro_version = 14;
+static const guint gmime_micro_version = 3;
 static const guint gmime_interface_age = 0;
 static const guint gmime_binary_age    = 0;
 
@@ -87,7 +86,7 @@ static const guint gmime_binary_age    = 0;
     (2 > (major) || \
      (2 == (major) && 1 > (minor)) || \
      (2 == (major) && 1 == (minor) && \
-      14 >= (micro)))
+      3 >= (micro)))
 
 
 /**
